package HTTP::Server::Sajax;
use strict;
use base qw'HTTP::Server::Simple::CGI Class::Accessor';
use Sajax;
use Template;
use Carp qw(croak);
use UNIVERSAL::require;
use HTTP::Server::Simple::Static;
use Data::Dumper;

use vars qw($VERSION);
$VERSION = '0.02';

__PACKAGE__->mk_accessors(qw(
    module 
    _template
    template 
    session_template
    static));

sub new {
    my ($package, %args) = @_;
    my $methods = delete $args{methods}
        or croak "No methods given to dispatch to (and I won't do magic autodispatch)";
    
    my $static = delete $args{static};
    croak "Static directory '$static' does not exist."
        unless -e $static;
    croak "Static directory '$static' is not a directory."
        unless -d $static;
    
    my $module = delete $args{module} || delete $args{object};
    my $template_file = delete $args{template};
    my $session_template_file = delete $args{session_template} || "sessions.tt";
    my $self = $package->SUPER::new(delete $args{port});
    $self->module($module);
    $self->static($static);
    $self->template($template_file);
    $self->session_template($session_template_file);

    my $config = {
        INCLUDE_PATH => './template',    # or list ref
        POST_CHOMP   => 1,               # cleanup whitespace
    };
    my $t = Template->new($config);
    $self->_template($t);

    $module->require
        unless ref $module;
    croak $@
        if $@;

    for (@$methods) {
        warn "Registering $_";
        my $n = $_;
        my $module_name = ref $self->module || $self->module;
    
        my $code = $self->module->can($n);
        croak "Method '$n' not found in module '$module_name'"
            unless $code;
        Sajax::rs_register($n, sub { $code->($self->module,@_) });
    };
    #warn $self->template();
    $self
};

sub handle_request {
    my ($self,$q) = @_;
    my $path = $q->url( -absolute => 1, -path_info => 1 );
    if ($path =~ m!^/session/(.+)$!) {
        $self->serve_ajax($q, $1);
    } elsif ($path =~ m!^/(?:session)?$!) {
        $self->serve_sessions($q);
    } elsif ($path =~ m!^/static/[\w.]+$!) {
        $self->serve_static( $q, $self->static . "/.." );
    } else {
        $self->serve_ajax($q);
    };
};

sub serve_ajax {
    my ($self,$q,@args) = @_;
    print "HTTP/1.0 200 OK\n";
    #print "Content-Type: text/html\n";

    if (my $rv = Sajax::rs_handle_client_request($q)) {
        # the rpc call was good
        #warn "Returning $rv";
        print "Content-Type: application/javascript\n";
        print "Content-Encoding: utf-8\n";
        print "\n";
        print $rv
    } else {
        # run the template
        print "Content-Type: text/html\n";
        print "Content-Encoding: utf-8\n";
        print "\n";
        $self->_template->process($self->template,
        {
            module    => $self->module,
            @args
            # XXX Should process/pass session here
            # resp. have a generic callback here
        });
        warn $self->_template->error
            if $self->_template->error;
    };
};

sub serve_sessions {
    my ($self,$q) = @_;
    $self->_template->process($self->session_template,
    {
            request   => $q,
            module    => $self->module,
    });
    warn $self->_template->error
        if $self->_template->error;
};
1;


=head1 AUTHOR

Written by Max Maischein E<lt>corion@cpan.orgE<gt>

Copyright Max Maischein 2005,2006; All Rights Reserved.

This program is free software; you can redistribute it
and/or modify it under the same terms as Perl itself.

=head1 SEE ALSO

L<Zaphod>

=cut
